#include <windows.h>   
#include <math.h>   

const float MISSING = (float) 3.4e38;

_declspec (dllexport) void ReflexIndicator ( float *Close, long Length, long N, float *output )
{	// (C) 2019 John F. Ehlers
	long i, count;
	double Slope=0.0, Sum=0.0, a1=0.0, b1=0.0, c1=0.0, c2=0.0, c3=0.0, MS=0.0, MS1=0.0, Reflex=0.0;
	double *Filt;
	double radiansconvertmult;

	radiansconvertmult = 3.1415926535 / 180.0;

	//Compute Super Smoother coefficients once
	a1 = exp(-1.414*3.14159 / (0.5*Length));
	b1 = 2*a1*cos(radiansconvertmult * (1.414 * 180 / (0.5*Length)));
	c2 = b1;
	c3 = -a1*a1;
	c1 = 1 - c2 - c3;

	Filt = malloc ( N * sizeof(double) );
	for ( i=0; i<N; i++ )
	{	if ( i < 2 )
		{	Filt[i] = Close[i];
		}
		else
		{	//SuperSmoother Filter
			Filt[i] = c1*(Close[i] + Close[i-1]) / 2 + c2*Filt[i-1] + c3*Filt[i-2];

			if ( i >= Length )
			{	//Length is assumed cycle period
				Slope = (Filt[i-Length] - Filt[i]) / Length;

				//Sum the differences
				Sum = 0.0;
				for ( count=1; count<=Length; count++ )
					Sum = Sum + (Filt[i] + count*Slope) - Filt[i-count];

				Sum = Sum / Length;

				//Normalize in terms of Standard Deviations
				MS = 0.04 * Sum * Sum + 0.96 * MS1;

				// NOTE: Following line of code in the easy language code, but charts in article match without this next line
				if ( MS != 0.0 ) Reflex = Sum / sqrt(MS);

				MS1 = MS;
				output[i] = (float) Reflex;
			}
		}

	}


	free(Filt);
}

_declspec (dllexport) void TrendflexIndicator ( float *Close, long Length, long N, float *output )
{	// (C) 2019 John F. Ehlers
	long i, count;
	double Sum=0.0, a1=0.0, b1=0.0, c1=0.0, c2=0.0, c3=0.0, MS=0.0, MS1=0.0, Trendflex=0.0;
	double *Filt;
	double radiansconvertmult;

	radiansconvertmult = 3.1415926535 / 180.0;

	//Compute Super Smoother coefficients once
	a1 = exp(-1.414*3.14159 / (0.5*Length));
	b1 = 2*a1*cos(radiansconvertmult * (1.414 * 180 / (0.5*Length)));
	c2 = b1;
	c3 = -a1*a1;
	c1 = 1 - c2 - c3;

	Filt = malloc ( N * sizeof(double) );
	for ( i=0; i<N; i++ )
	{	if ( i < 2 )
		{	Filt[i] = Close[i];
		}
		else
		{	//SuperSmoother Filter
			Filt[i] = c1*(Close[i] + Close[i-1]) / 2 + c2*Filt[i-1] + c3*Filt[i-2];

			if ( i >= Length )
			{
				//Sum the differences
				Sum = 0.0;
				for ( count=1; count<=Length; count++ )
					Sum = Sum + Filt[i] - Filt[i-count];

				Sum = Sum / Length;

				//Normalize in terms of Standard Deviations
				MS = 0.04 * Sum * Sum + 0.96 * MS1;

				// NOTE: Following line of code in the easy language code, but charts in article match without this next line
				if ( MS != 0.0 ) Trendflex = Sum / sqrt(MS);

				MS1 = MS;
			}

			output[i] = (float) Trendflex;
		}
	}

	free(Filt);
}

